import { AxiosResponse, AxiosRequestTransformer, AxiosResponseTransformer, Axios, AxiosRequestConfig } from 'axios';

interface ApiResponseError$1 {
  error?: any,
  errors?: { [key: string]: any }
}

type RawResponse$1<T> = AxiosResponse<T & ApiResponseError$1>

/**
 * 
 * @param data request data
 * @param headers request headers
 * @returns 
 */
declare const jsonRequest: AxiosRequestTransformer = (data, headers) => {
  if (typeof data !== 'string') {
    /**
     * Stringifies the data and sets a header if data is not a string
     */
    data = JSON.stringify(data)
    headers.setContentType('application/json')
  }
  console.log('jsonRequest', { data })
  return data
}

/**
 * 
 * @param data response data
 * @param headers response headers
 * @returns properly parsed data
 */
declare const jsonResponse: AxiosResponseTransformer = (data, headers) => {
  try {
    data = JSON.parse(data)
  } catch (error) {
  }
  // console.log('jsonResponse', { data })
  return data
}

/**
 * 
 * @param response the raw AxiosResponse which contains a `data` property
 * @returns Just the data, properly typed as T & {error, errors}
 */
declare function responseHandler<T = unknown>(response: RawResponse$1<T>): T & ApiResponseError$1 {
  let { data } = response
  // console.log('responseHandler', { data })
  if (data.error) console.error(data.error)
  if (data.errors) console.error(data.errors)
  return data
}

type AxiosPost = typeof Axios.prototype.post
type AxiosGet = typeof Axios.prototype.get
type AxiosDelete = typeof Axios.prototype.delete
type AxiosPatch = typeof Axios.prototype.patch
type ResponseInterceptor = Parameters<typeof Axios.prototype.interceptors.response.use>
type RequestInterceptor = Parameters<typeof Axios.prototype.interceptors.request.use>

declare class AxiosApi {
  private defaultConfig: AxiosRequestConfig = {
    responseType: 'json',
    transformRequest: jsonRequest,
    transformResponse: jsonResponse,
    validateStatus: (status) => status < 500
  }
  private axios: Axios;
  constructor(config?: AxiosRequestConfig) {
    this.axios = new Axios({ ...this.defaultConfig, ...config })
  }

  async post<T>(...args: Parameters<AxiosPost>) {
    return this.axios.post<ApiResponse<T>>(...args).then(responseHandler<T>)
  }
  async get<T>(...args: Parameters<AxiosGet>) {
    return this.axios.get<ApiResponse<T>>(...args).then(responseHandler<T>)
  }
  async delete<T>(...args: Parameters<AxiosDelete>) {
    return this.axios.post<ApiResponse<T>>(...args).then(responseHandler<T>)
  }
  async patch<T>(...args: Parameters<AxiosPatch>) {
    return this.axios.post<ApiResponse<T>>(...args).then(responseHandler<T>)
  }

  addRequestInterceptor(...options: RequestInterceptor) {
    this.axios.interceptors.request.use(...options)
  }
  addResponseInterceptor(...options: ResponseInterceptor) {
    this.axios.interceptors.response.use(...options)
  }
}

interface ApiResponseError {
  error?: any,
  errors?: { [key: string]: any }
}

type RawResponse<T> = AxiosResponse<T | ApiResponseError>

type ApiResponse<T> = T | ApiResponseError

export { ApiResponse, ApiResponseError, AxiosApi, AxiosDelete, AxiosGet, AxiosPatch, AxiosPost, RawResponse, RequestInterceptor, ResponseInterceptor, jsonRequest, jsonResponse, responseHandler };
